import logging
import threading


def _get_error_message(exc):
    if hasattr(exc, '__str__'):
        message = str(exc)
        return message[0].upper() + message[1:]
    return '\n'.join(exc.args),


def excepthook(exctype, exc, traceback, level=logging.CRITICAL, logger=None, context=None):
    """
    Error handler. Whatever happens in a plugin or component, if it looks like an exception, taste like an exception
    or somehow make me think it is an exception, I'll handle it.

    :param exc: the culprit
    :param trace: Hercule Poirot's logbook.
    :return: to hell
    """

    context = context or "thread {}".format(threading.get_ident())
    return (logger or logging.getLogger()).log(
        level,
        'Uncaught exception{}.'.format(' (in {})'.format(context) if context else ''),
        exc_info=(exctype, exc, traceback)
    )



import logging

from colorama import Fore

from mondrian import levels


class Filter(logging.Filter):
    def filter(self, record):
        record.color = ''
        record.spent = record.relativeCreated // 1000
        return True


class ColorFilter(Filter):
    def filter(self, record):
        super().filter(record)
        record.color = levels.COLORS.get(record.levelname, Fore.LIGHTWHITE_EX)
        return True



import logging
import sys

from mondrian import errors, filters, formatters, levels, term
from mondrian._version import __version__

__all__ = [
    '__version__',
    'errors',
    'filters',
    'formatters',
    'levels',
    'setup',
    'setup_excepthook',
    'term',
]


def setup_excepthook():
    """
    Replace default python exception hook with our own.

    """
    sys.excepthook = errors.excepthook


is_setup = False


def setup(*, colors=(not term.iswindows), excepthook=False):
    """
    Setup mondrian log handlers.

    :param colors: bool
    :param excepthook: bool
    """
    global is_setup

    if not is_setup:
        for level, name in levels.NAMES.items():
            logging.addLevelName(level, name)

        handler = logging.StreamHandler(sys.stderr)
        handler.setFormatter(formatters.Formatter())
        handler.addFilter(filters.ColorFilter() if colors else filters.Filter())

        logging.getLogger().addHandler(handler)
        logging.captureWarnings(True)

        is_setup = True

    if excepthook:
        setup_excepthook()


def getLogger(*args, colors=(not term.iswindows), excepthook=False, **kwargs):
    setup(colors=colors, excepthook=excepthook)
    return logging.getLogger(*args, **kwargs)



import os
import platform
import shlex
import struct
import subprocess
import sys

iswindows = (sys.platform == 'win32')


def _is_interactive_console():
    return sys.stdout.isatty()


def _is_jupyter_notebook():
    try:
        return get_ipython().__class__.__name__ == 'ZMQInteractiveShell'
    except NameError:
        return False


istty = _is_interactive_console()
isjupyter = _is_jupyter_notebook()


def _create_color_wrappers(symbol):
    if istty:
        from colorama import Fore, Back
        fg, bg, rfg, rbg = getattr(Fore, symbol), getattr(Back, symbol), Fore.RESET, Back.RESET
    else:
        fg, bg, rfg, rbg = '', '', '', ''

    def fg_wrapper(*args):
        return ''.join((fg, *args, rfg))

    def bg_wrapper(*args):
        return ''.join((bg, *args, rbg))

    return fg_wrapper, bg_wrapper


black, black_bg = _create_color_wrappers('BLACK')
red, red_bg = _create_color_wrappers('RED')
green, green_bg = _create_color_wrappers('GREEN')
yellow, yellow_bg = _create_color_wrappers('YELLOW')
blue, blue_bg = _create_color_wrappers('BLUE')
magenta, magenta_bg = _create_color_wrappers('MAGENTA')
cyan, cyan_bg = _create_color_wrappers('CYAN')
white, white_bg = _create_color_wrappers('WHITE')
reset, reset_bg = _create_color_wrappers('RESET')
lightblack, lightblack_bg = _create_color_wrappers('LIGHTBLACK_EX')
lightred, lightred_bg = _create_color_wrappers('LIGHTRED_EX')
lightgreen, lightgreen_bg = _create_color_wrappers('LIGHTGREEN_EX')
lightyellow, lightyellow_bg = _create_color_wrappers('LIGHTYELLOW_EX')
lightblue, lightblue_bg = _create_color_wrappers('LIGHTBLUE_EX')
lightmagenta, lightmagenta_bg = _create_color_wrappers('LIGHTMAGENTA_EX')
lightcyan, lightcyan_bg = _create_color_wrappers('LIGHTCYAN_EX')
lightwhite, lightwhite_bg = _create_color_wrappers('LIGHTWHITE_EX')


def bold(*args):
    if istty:
        from colorama import Style
        return ''.join((Style.BRIGHT, *args, Style.NORMAL))
    return ''.join(args)


CLEAR_EOL = '\033[0K'


def get_size():
    """ getTerminalSize()
     - get width and height of console
     - works on linux,os x,windows,cygwin(windows)
     originally retrieved from:
     http://stackoverflow.com/questions/566746/how-to-get-console-window-width-in-python
    """
    current_os = platform.system()
    tuple_xy = None
    if current_os == 'Windows':
        tuple_xy = _get_size_windows()
        if tuple_xy is None:
            tuple_xy = _get_size_tput()
            # needed for window's python in cygwin's xterm!
    if current_os in ['Linux', 'Darwin'] or current_os.startswith('CYGWIN'):
        tuple_xy = _get_size_linux()
    if tuple_xy is None:
        tuple_xy = (80, 25)  # default value
    return tuple_xy


def _get_size_windows():
    try:
        from ctypes import windll, create_string_buffer
        # stdin handle is -10
        # stdout handle is -11
        # stderr handle is -12
        h = windll.kernel32.GetStdHandle(-12)
        csbi = create_string_buffer(22)
        res = windll.kernel32.GetConsoleScreenBufferInfo(h, csbi)
        if res:
            (bufx, bufy, curx, cury, wattr, left, top, right, bottom, maxx,
             maxy) = struct.unpack("hhhhHhhhhhh", csbi.raw)
            sizex = right - left + 1
            sizey = bottom - top + 1
            return sizex, sizey
    except:
        pass


def _get_size_tput():
    # get terminal width
    # src: http://stackoverflow.com/questions/263890/how-do-i-find-the-width-height-of-a-terminal-window
    try:
        cols = int(subprocess.check_call(shlex.split('tput cols')))
        rows = int(subprocess.check_call(shlex.split('tput lines')))
        return (cols, rows)
    except:
        pass


def _get_size_linux():
    def ioctl_GWINSZ(fd):
        try:
            import fcntl
            import termios
            cr = struct.unpack('hh', fcntl.ioctl(fd, termios.TIOCGWINSZ, '1234'))
            return cr
        except:
            pass

    cr = ioctl_GWINSZ(0) or ioctl_GWINSZ(1) or ioctl_GWINSZ(2)
    if not cr:
        try:
            fd = os.open(os.ctermid(), os.O_RDONLY)
            cr = ioctl_GWINSZ(fd)
            os.close(fd)
        except:
            pass
    if not cr:
        try:
            cr = (os.environ['LINES'], os.environ['COLUMNS'])
        except:
            return None
    return int(cr[1]), int(cr[0])



import logging
import re
import textwrap
from traceback import format_exception

from colorama import Style
from mondrian.term import iswindows, lightblack, CLEAR_EOL, lightblack_bg, lightwhite

EOL = '\n' if iswindows else (Style.RESET_ALL + CLEAR_EOL + '\n')


class _Style(logging.PercentStyle):
    def __init__(self):
        tokens = (
            '%(color)s%(levelname)s',
            '%(spent)04d',
            '%(name)s',
        )

        sep = '|' if iswindows else (Style.RESET_ALL + lightblack(':'))
        self._fmt = sep.join(tokens) + lightblack(':') + ' %(message)s' + EOL.strip()


class Formatter(logging.Formatter):
    def __init__(self):
        self._style = _Style()
        self._fmt = self._style._fmt

    def formatException(self, excinfo):
        formatted_exception = format_exception(*excinfo)

        output = []
        stack_length = len(formatted_exception)
        for i, frame in enumerate(formatted_exception):
            if frame.startswith('  '):
                output.append(textwrap.indent('  ' + frame.strip(), lightblack('\u2502 ')))
            else:
                g = re.match('([a-zA-Z.]+): (.*)$', frame.strip(), flags=re.DOTALL)
                if g is not None:
                    etyp, emsg = g.group(1), g.group(2)
                    output.append(
                        lightblack('\u2514' if i + 1 == stack_length else '\u251c') +
                        lightblack_bg(lightwhite(' ' + etyp + ' ')) + ' ' +
                        lightwhite(textwrap.indent(str(emsg), ' ' * (len(etyp) + 4)).strip())
                    )
                else:
                    output.append(textwrap.indent(frame.strip(), lightblack('\u2502 ')))

        return EOL.join(output)



import logging

from colorama import Fore

DEBUG = 'DEBG'
INFO = 'INFO'
WARNING = 'WARN'
ERROR = 'ERR.'
CRITICAL = 'CRIT'

COLORS = {
    DEBUG: Fore.LIGHTCYAN_EX,
    INFO: Fore.LIGHTWHITE_EX,
    WARNING: Fore.LIGHTYELLOW_EX,
    ERROR: Fore.LIGHTRED_EX,
    CRITICAL: Fore.RED,
}

NAMES = {
    logging.DEBUG: DEBUG,
    logging.INFO: INFO,
    logging.WARNING: WARNING,
    logging.ERROR: ERROR,
    logging.CRITICAL: CRITICAL,
}
